@@ -31,33 +31,16 @@ static Scanner *scanner;
 
 static Book *book;
 
-static gboolean scanning = FALSE;
+static GTimer *log_timer;
 
-static gboolean first_autodetect = TRUE;
+static FILE *log_file;
+
+static gboolean debug = FALSE;
 
 
 static void
 update_scan_devices_cb (Scanner *scanner, GList *devices)
 {
-    if (first_autodetect) {
-        first_autodetect = FALSE;
-
-        if (!devices) {
-            gchar *selected;
-
-            selected = ui_get_selected_device (ui);
-            if (!selected)
-                ui_show_error (ui,
-                               /* Warning displayed when no scanners are detected */
-                               _("No scanners detected"),
-                               /* Hint to user on why there are no scanners detected */
-                               _("Please check your scanner is connected and powered on"),
-			       FALSE);
-            g_free (selected);
-        }
-    }
-
-    /* Add/update detected devices */
     ui_set_scan_devices (ui, devices);
 }
 
@@ -74,7 +57,7 @@ authorize_cb (Scanner *scanner, const gchar *resource)
 
 
 static Page *
-append_page (gboolean replace)
+append_page ()
 {
     Page *page;
     Orientation orientation = TOP_TO_BOTTOM;
@@ -82,8 +65,15 @@ append_page (gboolean replace)
     gchar *named_crop = NULL;
     gint width = 100, height = 100, dpi = 100, cx, cy, cw, ch;
 
-    /* Copy info from previous page */
+    /* Use current page if not used */
     page = book_get_page (book, -1);
+    if (page && !page_has_data (page)) {
+        ui_set_selected_page (ui, page);
+        page_start (page);
+        return page;
+    }
+  
+    /* Copy info from previous page */
     if (page) {
         orientation = page_get_orientation (page);
         width = page_get_scan_width (page);
@@ -97,9 +87,6 @@ append_page (gboolean replace)
         }
     }  
 
-    if (replace)
-        book_clear (book);
-
     page = book_append_page (book, width, height, dpi, orientation);
     if (do_crop) {
         if (named_crop)  {
@@ -109,7 +96,7 @@ append_page (gboolean replace)
         else
             page_set_custom_crop (page, cw, ch);
         page_move_crop (page, cx, cy);
-    }   
+    }
     ui_set_selected_page (ui, page);
     page_start (page);
   
@@ -117,6 +104,13 @@ append_page (gboolean replace)
 }
 
 
+static void
+scanner_new_page_cb (Scanner *scanner)
+{
+    append_page ();
+}
+
+
 static void
 scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
 {
@@ -125,14 +119,8 @@ scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
     g_debug ("Page is %d pixels wide, %d pixels high, %d bits per pixel",
              info->width, info->height, info->depth);
 
-    page = book_get_page (book, -1);
-  
     /* Add a new page */
-    if (page_get_scan_line (page) != 0) {
-        page = append_page (FALSE);
-    }
-
-    g_return_if_fail (page != NULL);
+    page = append_page ();
     page_set_scan_area (page, info->width, info->height, info->dpi);
 }
 
@@ -157,27 +145,31 @@ scanner_page_done_cb (Scanner *scanner)
 
 
 static void
-scanner_document_done_cb (Scanner *scanner)
-{
-    scanning = FALSE;
-    ui_set_scanning (ui, FALSE);
-    ui_set_have_scan (ui, TRUE);
-}
-
-
-static void
-scanner_failed_cb (Scanner *scanner, GError *error)
+remove_empty_page ()
 {
     Page *page;
 
     page = book_get_page (book, book_get_n_pages (book) - 1);
 
     /* Remove a failed page */
-    if (page_get_scan_line (page) == 0)
-        book_delete_page (book, page);
-    else
+    if (page_has_data (page))
         page_finish (page);
+    else
+        book_delete_page (book, page); 
+}
 
+
+static void
+scanner_document_done_cb (Scanner *scanner)
+{
+    remove_empty_page ();
+}
+
+
+static void
+scanner_failed_cb (Scanner *scanner, GError *error)
+{
+    remove_empty_page ();
     if (!g_error_matches (error, SCANNER_TYPE, SANE_STATUS_CANCELLED)) {
         ui_show_error (ui,
                        /* Title of error dialog when scan failed */
@@ -185,52 +177,46 @@ scanner_failed_cb (Scanner *scanner, GError *error)
                        error->message,
                        TRUE);
     }
-        
-    ui_set_scanning (ui, FALSE);
-    ui_set_have_scan (ui, TRUE);
 }
 
 
 static void
-scan_cb (SimpleScan *ui, const gchar *device, const gchar *profile_name, gboolean continuous, gboolean replace)
+scanner_scanning_changed_cb (Scanner *scanner)
 {
+    ui_set_scanning (ui, scanner_is_scanning (scanner));
+}
+
+
+static void
+scan_cb (SimpleScan *ui, const gchar *device, gint dpi, const gchar *profile_name, gboolean continuous)
+{
+    /* Default filename to use when saving document (and extension will be added, e.g. .jpg) */
+    const gchar *filename_prefix = _("Scanned Document");
     struct {
         const gchar *name;
-        gint dpi;
         ScanMode mode;
-        const gchar *file_name;
+        const gchar *extension;
     } profiles[] = 
     {
-        { "text", 200, SCAN_MODE_LINEART,
-          /* Default name for PDF documents */
-          _("Scanned Document.pdf") },
-        { "photo", 400, SCAN_MODE_COLOR,
-          /* Default name for JPEG documents */
-          _("Scanned Document.jpg") },
-        { "raw", 800, SCAN_MODE_COLOR,
-          /* Default name for PNG documents */
-          _("Scanned Document.png") },
-        { NULL, 75, SCAN_MODE_COLOR,
-          /* Default name for JPEG documents */
-          _("Scanned Document.jpg") }                
+        { "text",  SCAN_MODE_LINEART, "pdf" },
+        { "photo", SCAN_MODE_COLOR,   "jpg" },
+        { NULL,    SCAN_MODE_COLOR,   "jpg" }
     };
-    Page *page;
+    gchar *filename;
     gint i;
 
-    g_debug ("Requesting scan of type %s from device '%s'", profile_name, device);
+    g_debug ("Requesting scan of type %s at %d dpi from device '%s'", profile_name, dpi, device);
 
     /* Find this profile */
     for (i = 0; profiles[i].name && strcmp (profiles[i].name, profile_name) != 0; i++);
 
-    if (!scanning)
-        page = append_page (replace);
+    if (!scanner_is_scanning (scanner))
+        append_page ();
 
-    scanning = TRUE;
-    ui_set_have_scan (ui, FALSE);
-    ui_set_scanning (ui, TRUE);
- 
-    ui_set_default_file_name (ui, profiles[i].file_name);
-    scanner_scan (scanner, device, profiles[i].dpi, profiles[i].mode, 8, continuous);
+    filename = g_strdup_printf ("%s.%s", filename_prefix, profiles[i].extension);
+    ui_set_default_file_name (ui, filename);
+    g_free (filename);
+    scanner_scan (scanner, device, dpi, profiles[i].mode, 8, continuous);
 }
 
 
@@ -282,59 +268,91 @@ save_cb (SimpleScan *ui, const gchar *uri)
 }
 
 
-static void
-email_cb (SimpleScan *ui)
+static gchar *
+get_temporary_filename (const gchar *extension)
 {
-    gint i;
-    gchar *dir, *path = NULL, *uri = NULL;
+    gint fd;
+    gchar *filename, *path;
+    GError *error = NULL;
 
-    // TODO: Delete old files on startup
+    /* NOTE: I'm not sure if this is a 100% safe strategy to use g_file_open_tmp(), close and
+     * use the filename but it appears to work in practise */
 
-    /* Save in the temporary dir */
-    dir = g_build_filename (g_get_user_cache_dir (), "simple-scan", "email", NULL);
-    g_mkdir_with_parents (dir, 0700);
+    filename = g_strdup_printf ("scanned-document-XXXXXX.%s", extension);
+    fd = g_file_open_tmp (filename, &path, &error);
+    g_free (filename);
+    if (fd < 0) {
+        g_warning ("Error saving email attachment: %s", error->message);
+        g_clear_error (&error);
+        return NULL;
+    }
+    close (fd);
 
-    for (i = 0; ; i++) {
-        GString *filename;
-        GError *error = NULL;
+    return path;
+}
 
-        filename = g_string_new ("");
-        g_string_printf (filename, "scan-%d-%d.pdf", getpid (), i);
 
-        g_free (path);
-        path = g_build_filename (dir, filename->str, NULL);
-        g_string_free (filename, TRUE);
+static void
+email_cb (SimpleScan *ui, const gchar *profile)
+{
+    gboolean saved = FALSE;
+    GError *error = NULL;
+    GString *command_line;
+  
+    command_line = g_string_new ("xdg-email");
 
-        g_free (uri);
-        uri = g_filename_to_uri (path, NULL, NULL);
+    /* Save text files as PDFs */
+    if (strcmp (profile, "text") == 0) {
+        gchar *path, *uri;
 
-        if (book_save_pdf (book, uri, &error)) {
-            GString *command_line;
-
-            command_line = g_string_new ("");
-            g_string_printf (command_line, "xdg-email --attach %s", path);
-            g_debug ("Launchind email client: %s", command_line->str);
-            g_spawn_command_line_async (command_line->str, &error);
-
-            if (error) {
-                g_warning ("Unable to start email: %s", error->message);
-                g_clear_error (&error);
-            }
-            g_string_free (command_line, TRUE);
-	    break;
+        /* Open a temporary file */
+        path = get_temporary_filename ("pdf");
+        if (path) {
+            uri = g_filename_to_uri (path, NULL, NULL);
+            saved = book_save_pdf (book, uri, &error);
+            g_string_append_printf (command_line, " --attach %s", path);
+            g_free (path);
+            g_free (uri);
         }
-        else {
-            if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
-                g_warning ("Unable to save email PDF: %s", error->message);
-                g_clear_error (&error);
+    }
+    else {
+        gint i;
+
+        for (i = 0; i < book_get_n_pages (book); i++) {
+            gchar *path, *uri;
+
+            path = get_temporary_filename ("jpeg");
+            if (!path) {
+                saved = FALSE;
                 break;
             }
+
+            uri = g_filename_to_uri (path, NULL, NULL);
+            saved = page_save_jpeg (book_get_page (book, i), uri, &error);
+            g_string_append_printf (command_line, " --attach %s", path);
+            g_free (path);
+            g_free (uri);
+          
+            if (!saved)
+                break;
         }
     }
 
-    g_free (path);
-    g_free (uri);   
-    g_free (dir);
+    if (saved) {
+        g_debug ("Launchind email client: %s", command_line->str);
+        g_spawn_command_line_async (command_line->str, &error);
+
+        if (error) {
+            g_warning ("Unable to start email: %s", error->message);
+            g_clear_error (&error);
+        }
+    }
+    else {
+        g_warning ("Unable to save email file: %s", error->message);
+        g_clear_error (&error);
+    }
+
+    g_string_free (command_line, TRUE);
 }
 
 
@@ -394,11 +412,46 @@ usage(int show_gtk)
 
 static void
 log_cb (const gchar *log_domain, GLogLevelFlags log_level,
-	const gchar *message, gpointer data)
+        const gchar *message, gpointer data)
 {
-    if (log_level & G_LOG_LEVEL_DEBUG)
-       return;
-    g_log_default_handler (log_domain, log_level, message, data);
+    /* Log everything to a file */
+    if (log_file) {
+        const gchar *prefix;
+
+        switch (log_level & G_LOG_LEVEL_MASK) {
+        case G_LOG_LEVEL_ERROR:
+            prefix = "ERROR:";
+            break;
+        case G_LOG_LEVEL_CRITICAL:
+            prefix = "CRITICAL:";
+            break;
+        case G_LOG_LEVEL_WARNING:
+            prefix = "WARNING:";
+            break;
+        case G_LOG_LEVEL_MESSAGE:
+            prefix = "MESSAGE:";
+            break;
+        case G_LOG_LEVEL_INFO:
+            prefix = "INFO:";
+            break;
+        case G_LOG_LEVEL_DEBUG:
+            prefix = "DEBUG:";
+            break;
+        default:
+            prefix = "LOG:";
+            break;
+        }
+
+        fprintf (log_file, "[%+.2fs] %s %s\n", g_timer_elapsed (log_timer, NULL), prefix, message);
+    }
+
+    /* Only show debug if requested */
+    if (log_level & G_LOG_LEVEL_DEBUG) {
+        if (debug)
+            g_log_default_handler (log_domain, log_level, message, data);
+    }
+    else
+        g_log_default_handler (log_domain, log_level, message, data);    
 }
 
 
@@ -406,7 +459,6 @@ static void
 get_options (int argc, char **argv)
 {
     int i;
-    gboolean debug = FALSE;
 
     for (i = 1; i < argc; i++) {
         char *arg = argv[i];
@@ -437,10 +489,7 @@ get_options (int argc, char **argv)
             }
             default_device = arg;
         }
-    }
-   
-    if (!debug)
-        g_log_set_default_handler (log_cb, NULL);
+    }   
 }
 
 
@@ -455,16 +504,30 @@ main (int argc, char **argv)
 {
     GUdevClient *udev_client;
     const char *udev_subsystems[] = { "usb", NULL };
+    gchar *path;
+
+    g_thread_init (NULL);
+
+    /* Log to a file */
+    log_timer = g_timer_new ();
+    path = g_build_filename (g_get_user_cache_dir (), "simple-scan", NULL);
+    g_mkdir_with_parents (path, 0700);
+    g_free (path);
+    path = g_build_filename (g_get_user_cache_dir (), "simple-scan", "simple-scan.log", NULL);
+    log_file = fopen (path, "w");
+    g_free (path);
+    g_log_set_default_handler (log_cb, NULL);
 
     bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
    
-    g_thread_init (NULL);
     gtk_init (&argc, &argv);
 
     get_options (argc, argv);
   
+    g_debug ("Starting Simple Scan %s, PID=%i", VERSION, getpid ());
+  
     ui = ui_new ();
     book = ui_get_book (ui);
     g_signal_connect (ui, "start-scan", G_CALLBACK (scan_cb), NULL);
@@ -476,11 +539,13 @@ main (int argc, char **argv)
     scanner = scanner_new ();
     g_signal_connect (G_OBJECT (scanner), "update-devices", G_CALLBACK (update_scan_devices_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "authorize", G_CALLBACK (authorize_cb), NULL);
+    g_signal_connect (G_OBJECT (scanner), "expect-page", G_CALLBACK (scanner_new_page_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "got-page-info", G_CALLBACK (scanner_page_info_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "got-line", G_CALLBACK (scanner_line_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "page-done", G_CALLBACK (scanner_page_done_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "document-done", G_CALLBACK (scanner_document_done_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "scan-failed", G_CALLBACK (scanner_failed_cb), NULL);
+    g_signal_connect (G_OBJECT (scanner), "scanning-changed", G_CALLBACK (scanner_scanning_changed_cb), NULL);
 
     udev_client = g_udev_client_new (udev_subsystems);
     g_signal_connect (udev_client, "uevent", G_CALLBACK (on_uevent), NULL);
